@@ -204,11 +204,12 @@ struct _MetaDisplay
    */
   guint mouse_mode : 1;
 
-  /* Helper var for strict focus for terminals; only relevant if the focus
-   * window is a terminal.  Typically, we don't allow new windows to take
-   * focus away from a terminal, but if the user explicitly did something
-   * that should allow a different window to gain focus (e.g. global
-   * keybinding or clicking on a dock), then we will allow the transfer.
+  /* Helper var used when focus_new_windows setting is 'strict'; only
+   * relevant in 'strict' mode and if the focus window is a terminal.
+   * In that case, we don't allow new windows to take focus away from
+   * a terminal, but if the user explicitly did something that should
+   * allow a different window to gain focus (e.g. global keybinding or
+   * clicking on a dock), then we will allow the transfer.
    */
   guint allow_terminal_deactivation : 1;
 
@@ -218,18 +219,13 @@ struct _MetaDisplay
   guint error_trap_synced_at_last_pop : 1;
   MetaEventQueue *events;
   GSList *screens;
+  MetaScreen *active_screen;
   GHashTable *window_ids;
   int error_traps;
   int (* error_trap_handler) (Display     *display,
                               XErrorEvent *error);  
   int server_grab_count;
 
-  /* for double click */
-  Time last_button_time;
-  Window last_button_xwindow;
-  int last_button_num;
-  guint is_double_click : 1;
-
   /* serials of leave/unmap events that may
    * correspond to an enter event we should
    * ignore
@@ -269,8 +265,8 @@ struct _MetaDisplay
   MetaRectangle grab_wireframe_rect;
   MetaRectangle grab_wireframe_last_xor_rect;
   MetaRectangle grab_initial_window_pos;
-  int         grab_initial_x, grab_initial_y;
-  gboolean    grab_threshold_movement_reached;
+  int         grab_initial_x, grab_initial_y;  /* These are only relevant for */
+  gboolean    grab_threshold_movement_reached; /* raise_on_click == FALSE.    */
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
   Time        grab_motion_notify_time;
@@ -278,7 +274,8 @@ struct _MetaDisplay
   int         grab_wireframe_last_display_height;
   GList*      grab_old_window_stacking;
   MetaEdgeResistanceData *grab_edge_resistance_data;
-  
+  unsigned int grab_last_user_action_was_snap;
+
   /* we use property updates as sentinels for certain window focus events
    * to avoid some race conditions on EnterNotify events
    */
@@ -286,6 +283,7 @@ struct _MetaDisplay
 
 #ifdef HAVE_XKB
   int         xkb_base_event_type;
+  Time        last_bell_time;
 #endif
 #ifdef HAVE_XSYNC
   /* alarm monitoring client's _NET_WM_SYNC_REQUEST_COUNTER */
@@ -298,8 +296,8 @@ struct _MetaDisplay
   int             n_screen_bindings;
   MetaKeyBinding *window_bindings;
   int             n_window_bindings;
-  unsigned int min_keycode;
-  unsigned int max_keycode;
+  int             min_keycode;
+  int             max_keycode;
   KeySym *keymap;
   int keysyms_per_keycode;
   XModifierKeymap *modmap;
@@ -348,7 +346,6 @@ struct _MetaDisplay
   int render_error_base;
 #endif
 #ifdef HAVE_XSYNC
-  unsigned int grab_last_user_action_was_snap;
   unsigned int have_xsync : 1;
 #define META_DISPLAY_HAS_XSYNC(display) ((display)->have_xsync)
 #else
@@ -381,13 +378,13 @@ struct _MetaDisplay
  * the result.
  */
 #define XSERVER_TIME_IS_BEFORE_ASSUMING_REAL_TIMESTAMPS(time1, time2) \
-  ( (( time1 < time2 ) && ( time2 - time1 < ((guint32)-1)/2 )) ||     \
-    (( time1 > time2 ) && ( time1 - time2 > ((guint32)-1)/2 ))        \
+  ( (( (time1) < (time2) ) && ( (time2) - (time1) < ((guint32)-1)/2 )) ||     \
+    (( (time1) > (time2) ) && ( (time1) - (time2) > ((guint32)-1)/2 ))        \
   )
 #define XSERVER_TIME_IS_BEFORE(time1, time2)                          \
-  ( time1 == 0 ||                                                     \
+  ( (time1) == 0 ||                                                     \
     (XSERVER_TIME_IS_BEFORE_ASSUMING_REAL_TIMESTAMPS(time1, time2) && \
-     time2 != 0)                                                      \
+     (time2) != 0)                                                      \
   )
 
 gboolean      meta_display_open                (const char  *name);
@@ -400,7 +397,6 @@ MetaScreen*   meta_display_screen_for_xwindow  (MetaDisplay *display,
                                                 Window       xindow);
 void          meta_display_grab                (MetaDisplay *display);
 void          meta_display_ungrab              (MetaDisplay *display);
-gboolean      meta_display_is_double_click     (MetaDisplay *display);
 
 void          meta_display_unmanage_screen     (MetaDisplay *display,
                                                 MetaScreen  *screen);
@@ -498,7 +494,8 @@ gboolean meta_display_window_has_pending_pings (MetaDisplay        *display,
 typedef enum
 {
   META_TAB_LIST_NORMAL,
-  META_TAB_LIST_DOCKS
+  META_TAB_LIST_DOCKS,
+  META_TAB_LIST_GROUP
 } MetaTabList;
 
 typedef enum
