@@ -53,7 +53,8 @@ typedef enum
   META_PREF_REDUCED_RESOURCES,
   META_PREF_GNOME_ACCESSIBILITY,
   META_PREF_CURSOR_THEME,
-  META_PREF_CURSOR_SIZE
+  META_PREF_CURSOR_SIZE,
+  META_PREF_COMPOSITING_MANAGER
 } MetaPreference;
 
 typedef void (* MetaPrefsChangedFunc) (MetaPreference pref,
@@ -100,6 +101,7 @@ void        meta_prefs_change_workspace_name (int         i,
 
 const char* meta_prefs_get_cursor_theme      (void);
 int         meta_prefs_get_cursor_size       (void);
+gboolean    meta_prefs_get_compositing_manager (void);
 
 /* Screen bindings */
 #define META_KEYBINDING_WORKSPACE_1              "switch_to_workspace_1"
@@ -118,10 +120,14 @@ int         meta_prefs_get_cursor_size       (void);
 #define META_KEYBINDING_WORKSPACE_RIGHT          "switch_to_workspace_right"
 #define META_KEYBINDING_WORKSPACE_UP             "switch_to_workspace_up"
 #define META_KEYBINDING_WORKSPACE_DOWN           "switch_to_workspace_down"
+#define META_KEYBINDING_SWITCH_GROUP             "switch_group"
+#define META_KEYBINDING_SWITCH_GROUP_BACKWARD    "switch_group_backward"
 #define META_KEYBINDING_SWITCH_WINDOWS           "switch_windows"
 #define META_KEYBINDING_SWITCH_WINDOWS_BACKWARD  "switch_windows_backward"
 #define META_KEYBINDING_SWITCH_PANELS            "switch_panels"
 #define META_KEYBINDING_SWITCH_PANELS_BACKWARD   "switch_panels_backward"
+#define META_KEYBINDING_CYCLE_GROUP              "cycle_group"
+#define META_KEYBINDING_CYCLE_GROUP_BACKWARD     "cycle_group_backward"
 #define META_KEYBINDING_CYCLE_WINDOWS            "cycle_windows"
 #define META_KEYBINDING_CYCLE_WINDOWS_BACKWARD   "cycle_windows_backward"
 #define META_KEYBINDING_CYCLE_PANELS             "cycle_panels"
@@ -219,10 +225,14 @@ typedef enum _MetaKeyBindingAction
   META_KEYBINDING_ACTION_WORKSPACE_RIGHT,
   META_KEYBINDING_ACTION_WORKSPACE_UP,
   META_KEYBINDING_ACTION_WORKSPACE_DOWN,
+  META_KEYBINDING_ACTION_SWITCH_GROUP,
+  META_KEYBINDING_ACTION_SWITCH_GROUP_BACKWARD,
   META_KEYBINDING_ACTION_SWITCH_WINDOWS,
   META_KEYBINDING_ACTION_SWITCH_WINDOWS_BACKWARD,
   META_KEYBINDING_ACTION_SWITCH_PANELS,
   META_KEYBINDING_ACTION_SWITCH_PANELS_BACKWARD,
+  META_KEYBINDING_ACTION_CYCLE_GROUP,
+  META_KEYBINDING_ACTION_CYCLE_GROUP_BACKWARD,
   META_KEYBINDING_ACTION_CYCLE_WINDOWS,
   META_KEYBINDING_ACTION_CYCLE_WINDOWS_BACKWARD,
   META_KEYBINDING_ACTION_CYCLE_PANELS,
@@ -246,9 +256,21 @@ typedef enum _MetaKeyBindingAction
 
 typedef struct
 {
-  const char   *name;
-  unsigned int  keysym;
+  unsigned int keysym;
   MetaVirtualModifier modifiers;
+} MetaKeyCombo;
+
+typedef struct
+{
+  const char   *name;
+  /* a list of MetaKeyCombos. Each of them is bound to
+   * this keypref. If one has keysym==modifiers==0, it is
+   * ignored. For historical reasons, the first entry is
+   * governed by the pref FOO and the remainder are
+   * governed by the pref FOO_list.
+   */
+  GSList *bindings;
+
   /* for keybindings that can have shift or not like Alt+Tab */
   gboolean      add_shift;
 } MetaKeyPref;
