@@ -239,6 +239,9 @@ struct _MetaWindow
   /* Are we in meta_window_free()? */
   guint unmanaging : 1;
 
+  /* Are we in meta_window_new()? */
+  guint constructing : 1;
+  
   /* Are we in the calc_showing queue? */
   guint calc_showing_queued : 1;
 
@@ -270,10 +273,8 @@ struct _MetaWindow
   guint transient_parent_is_root_window : 1;
 
   /* Info on which props we got our attributes from */
-  guint using_net_wm_name              : 1; /* vs. plain wm_name */
-  guint using_net_wm_visible_name      : 1; /* tracked so we can clear it */
-  guint using_net_wm_icon_name         : 1; /* vs. plain wm_icon_name */
-  guint using_net_wm_visible_icon_name : 1; /* tracked so we can clear it */
+  guint using_net_wm_name : 1; /* vs. plain wm_name */
+  guint using_net_wm_icon_name : 1; /* vs. plain wm_icon_name */
 
   /* has a shape mask */
   guint has_shape : 1;
@@ -541,6 +542,8 @@ gboolean meta_window_same_application (MetaWindow *window,
   (((w)->input || (w)->take_focus ) && META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w) && (!(w)->skip_taskbar))
 #define META_WINDOW_IN_DOCK_TAB_CHAIN(w) \
   (((w)->input || (w)->take_focus) && ! META_WINDOW_IN_NORMAL_TAB_CHAIN_TYPE (w))
+#define META_WINDOW_IN_GROUP_TAB_CHAIN(w, g) \
+  (((w)->input || (w)->take_focus) && (!g || meta_window_get_group(w)==g))
 
 void meta_window_refresh_resize_popup (MetaWindow *window);
 
@@ -572,7 +575,8 @@ gboolean meta_window_get_icon_geometry (MetaWindow    *window,
 
 const char* meta_window_get_startup_id (MetaWindow *window);
 
-void meta_window_recalc_features (MetaWindow *window);
+void meta_window_recalc_features    (MetaWindow *window);
+void meta_window_recalc_window_type (MetaWindow *window);
 
 void meta_window_queue_update_icon (MetaWindow *window);
 
@@ -582,4 +586,7 @@ void meta_window_stack_just_below (MetaWindow *window,
 void meta_window_set_user_time (MetaWindow *window,
                                 Time        timestamp);
 
+void meta_window_set_demands_attention (MetaWindow *window);
+
+void meta_window_unset_demands_attention (MetaWindow *window);
 #endif
